;
;  disk write protect program
;
; This program re-directs interrupt 13 if a request
; is made to write to a disk which has been set in the table
;
; It uses ah to change a disks status
;     ah = ff = status
;     ah = fe = write protect
;     ah = fd = un-write protect
;
; al says what disk is to be write protected
;  al = 0 = a
;  al = 1 = b
;  al = 2 = c
;
cseg    segment
wprot   proc  far
assume  cs:cseg,ds:cseg,es:cseg,ss:cseg
        org  100h
start:  jmp setup
bio1    dw 0
bio2    dw 0
d251    dw 0
d252    dw 0
;
writeon db 0  ; tells if write protect is active
dtable  db 256 dup(0)          ; write protect table
res     db 0
int25:
        sti
        cmp  ah,0feh
        je   jmpreq25
        cmp  ah,0fdh
        je   jmpreq25
        cmp  ah,0
        je   jmpgo25
        cmp  ah,8
        je   jmpgo25
        cmp  ah,9
        je   jmpgo25
        cmp  ah,12h
        je   jmpgo25
        cmp  ah,13h
        je   jmpgo25
        cmp  ah,14h
        je   jmpgo25
        cmp  ah,19h
        je   jmpgo25
        jmp  action2
jmpgo25
        jmp  dword ptr cs:d251 ; call bios interrupt 25h
action25:
        call  action1
  ; we have set the flags and return
        jnc   goback25
        ret   ; leave flags in stack
goback25:
        jmp  dword ptr cs:d251 ; call dos interrupt 25h
int13:
        sti
        cmp  ah,'K'
        je   jmpreq
        cmp  ah,0feh
        je   jmpreq
        cmp  ah,0fdh
        je   jmpreq
        cmp  ah,0
        je   jmpgo
        cmp  ah,8
        je   jmpgo
        cmp  ah,9
        je   jmpgo
        cmp  ah,12h
        je   jmpgo
        cmp  ah,13h
        je   jmpgo
        cmp  ah,14h
        je   jmpgo
        cmp  ah,19h
        je   jmpgo
        jmp  action
jmpgo:
        jmp  dword ptr cs:bio1 ; call bios interrupt 13h
jmpreq:
        call request
        ret  2
action:
        call  action1
  ; we have set the flags and return
        jnc   goback2
        ret  2
goback2:
        jmp  dword ptr cs:bio1 ; call bios interrupt 13h
wprot   endp
action1 proc near
        push bx
        push cx
        push ds
        push si
        push di
        push bp
        push dx
        push es
 ; we have to check the if the disk is in the table
        mov  bh,0
        mov  bl,dl
cont:
        cmp  byte ptr cs:[dtable][bx],0
        jne  hitit
        jmp  goback
hitit:
        mov  ax,40
        mov  es,ax
        mov  byte ptr es:[41h],06h  ; set diskette status
        mov   ah,6
        mov   al,0
        cmp   ah,1
        stc
        pop  es
        pop  dx
        pop  bp
        pop  di
        pop  si
        pop  ds
        pop  cx
        pop  bx
        ret
request:
        push bx
        push cx
        push ds
        push si
        push di
        push bp
        push dx
        push es
; we have now a request for action
;     ah = 'K'= status
;     ah = fe = write protect
;     ah = fd = un-write protect
;     al contains a disk number
      cmp  ah,'K'
      je   status
      cmp  ah,0feh
      je   protect
unprotect:
      mov  bh,0
      mov  bl,al
      mov  byte ptr cs:[dtable][bx],0
      jmp  status
protect:
      mov  bh,0
      mov  bl,al
      mov  byte ptr cs:[dtable][bx],1
      jmp  status
status:
; status just means that the program is resident
        pop  es
        pop  dx
        pop  bp
        pop  di
        pop  si
        pop  ds
        pop  cx
        pop  bx
        stc
        ret
goback:
        pop  es
        pop  dx
        pop  bp
        pop  di
        pop  si
        pop  ds
        pop  cx
        pop  bx
        clc
        ret
action1 endp
end_prog:
setup:
        mov   ah,'K'
        mov   al,0
        int   13h
        jc    fixint
        popf
        jmp   resident
fixint:
; replace interrupts
        popf
        push  es
        mov   ax,3513h
        int   21h
        mov   bio1,bx
        mov   ax,es
        mov   bio2,ax
        mov   dx,offset int13
        mov   ax,2513h
        int   21h
;       mov   ax,3525h
;       int   21h
;       mov   d251,bx
;       mov   ax,es
;       mov   d252,ax
;       mov   dx,offset int25
;       mov   ax,2525h
;       int   21h
        mov   res,1
        pop   es
resident:
; now we have to inspect the commandline in order to determine things
        mov   ah,0
        mov   al,byte ptr ds:[5ch]
        cmp   al,1
        jl   diskng
        cmp   al,9
        jg   diskng
        jmp  diskok
diskng:
        mov   ah,9
        mov   dx,offset err1
        int   21h
        jmp   exit
diskok:
        mov   bx,ax
; now which disk?
        int   11h
        and   ah,3
        cmp   bl,ah
        jg    hard
        jmp   floppy
hard:
        add  bx,80h    ; hard disk use 80 in dl to use int 13
                       ; don't know if 81 is second hard disk of not
                       ; I am guesing that it does
                       ; I'm sure someone will call and tell me
        sub  bl,ah
floppy:
        mov   cl,byte ptr ds:[80h]
        mov   ch,0
        cmp   cl,2
        jl    diskng
        mov   di,81h
        mov   al,'/'
repne   scasb
        jne   diskng
        mov   si,di
        lodsb
        cmp   al,'U'
        je    unp
        cmp   al,'u'
        je    unp
        cmp   al,'P'
        je    prot
        cmp   al,'p'
        je    prot
        jmp   diskng
unp:
        mov   ah,0
        mov   al,bl
        dec   ax
        mov   ah,0fdh
        int   13h
        popf
        jmp   exit
prot:
        mov   ah,0
        mov   al,bl
        dec   ax
        mov   ah,0feh
        int   13h
        popf
        jmp   exit
exit:
        push   cs
        pop    ds
; leave and remain resident
        cmp   res,0
        je    int20
        mov   dx,offset end_prog
        int   27h
int20:
        int   20h
err1    db  'Graham Systems Disk Write Protect V1.1',13,10
        db  'Copyright 1985,1987 by Keith P. Graham',13,10
        db  '                       238 Germonds Rd.',13,10
        db  '                       W. Nyack, NY 10994',13,10
        db  'Released from PC-Rockland BBS (914) 353-2176',13,10,10
        db  'The Proper form of the WPROT.COM is:      ',13,10
        db  '  WPROT X: /y       ',13,10
        db  ' Where X is a valid disk drive letter such as C:',13,10
        db  ' and y is an option. ',13,10
        db  '  /P = Protect the disk',13,10
        db  '  /U = Un-Protect - restores to actual status of disk'
        db  13,10,10,'$'
cseg    ends
end     start
